/**
 * Base game map script - Produces widely varied continents.
 * @packageDocumentation
 */
console.log("Generating using script fractal.ts");
import { assignStartPositions, chooseStartSectors } from '/base-standard/maps/assign-starting-plots.js';
import { addMountains, addHills, buildRainfallMap, generateLakes } from '/base-standard/maps/elevation-terrain-generator.js';
import { addFeatures, designateBiomes } from '/base-standard/maps/feature-biome-generator.js';
import * as globals from '/base-standard/maps/map-globals.js';
import * as utilities from '/base-standard/maps/map-utilities.js';
import { addNaturalWonders } from '/base-standard/maps/natural-wonder-generator.js';
import { generateResources } from '/randomContinents/maps/resources.js';
import { addVolcanoes } from '/base-standard/maps/volcano-generator.js';
import { assignAdvancedStartRegions } from '/base-standard/maps/assign-advanced-start-region.js';
import { generateDiscoveries } from '/base-standard/maps/discovery-generator.js';
import { generateSnow, dumpPermanentSnow } from '/base-standard/maps/snow-generator.js';
import { dumpStartSectors, dumpContinents, dumpTerrain, dumpElevation, dumpRainfall, dumpBiomes, dumpFeatures, dumpResources, dumpNoisePredicate } from '/base-standard/maps/map-debug-helpers.js';
import { getBorderDistanceX, getBorderDistanceY, createRandomBorder, random01, getContinentFromBorders } from '/randomContinents/maps/hemisphere.js';

function requestMapData(initParams) {
    console.log(initParams.width);
    console.log(initParams.height);
    console.log(initParams.topLatitude);
    console.log(initParams.bottomLatitude);
    console.log(initParams.wrapX);
    console.log(initParams.wrapY);
    console.log(initParams.mapSize);
    engine.call("SetMapInitData", initParams);
}

function generateMap() {
    console.log("Generating a map!");
    console.log(`Age - ${GameInfo.Ages.lookup(Game.age).AgeType}`);
    let iWidth = GameplayMap.getGridWidth();
    let iWidth2 = iWidth / 2;
    let iHeight = GameplayMap.getGridHeight();
    let uiMapSize = GameplayMap.getMapSize();
    let startPositions = [];
    let mapInfo = GameInfo.Maps.lookup(uiMapSize);
    if (mapInfo == null)
        return;
    // Establish continent boundaries
    let iOceanWaterColumns = globals.g_OceanWaterColumns + 4;
    let marginX = iWidth2 / 10;
    let marginY = iWidth2 / 20;
    let westStartRegion = {
        west: marginX,
        east: iWidth2 - marginX,
        south: marginY,
        north: iHeight - marginY,
        continent: 0
    };
    let eastStartRegion = {
        west: iWidth2 + marginX,
        east: iWidth - marginX,
        south: marginY,
        north: iHeight - marginY,
        continent: 0
    };

    let startSectors;
    let iNumPlayers1 = mapInfo.PlayersLandmass1;
    let iNumPlayers2 = mapInfo.PlayersLandmass2;
    let iNumNaturalWonders = Math.round(mapInfo.NumNaturalWonders * 1);
    let iTilesPerLake = mapInfo.LakeGenerationFrequency;
    let iStartSectorRows = mapInfo.StartSectorRows;
    let iStartSectorCols = mapInfo.StartSectorCols;
    let iRandom = TerrainBuilder.getRandomNumber(2, "East or West");
    console.log("Random Hemisphere: " + iRandom);
    if (iRandom == 1) {
        let iNum1 = iNumPlayers1;
        let iNum2 = iNumPlayers2;
        iNumPlayers1 = iNum2;
        iNumPlayers2 = iNum1;
    }
    let seed = GameplayMap.getRandomSeed(); // can use any seed you want for different noises
    let bHumanNearEquator = utilities.needHumanNearEquator();
    startSectors = chooseStartSectors(iNumPlayers1, iNumPlayers2, iStartSectorRows, iStartSectorCols, bHumanNearEquator);
    let extraMapInfo = new Array(iWidth * iHeight);
    createLandmasses(iWidth, iHeight, westStartRegion, eastStartRegion, iStartSectorRows, iStartSectorCols, startSectors, extraMapInfo);
    TerrainBuilder.validateAndFixTerrain();
    utilities.adjustOceanPlotTags(iNumPlayers1 > iNumPlayers2);
    AreaBuilder.recalculateAreas();
    TerrainBuilder.stampContinents();
    addMountains(iWidth, iHeight);
    addVolcanoes(iWidth, iHeight);
    generateLakes(iWidth, iHeight, iTilesPerLake);
    AreaBuilder.recalculateAreas();
    TerrainBuilder.buildElevation();
    addHills(iWidth, iHeight);
    buildRainfallMap(iWidth, iHeight);
    TerrainBuilder.modelRivers(5, 15, globals.g_NavigableRiverTerrain);
    TerrainBuilder.validateAndFixTerrain();
    TerrainBuilder.defineNamedRivers();
    designateBiomes(iWidth, iHeight);
    addNaturalWonders(iWidth, iHeight, iNumNaturalWonders);
    TerrainBuilder.addFloodplains(4, 10);
    addFeatures(iWidth, iHeight);
    TerrainBuilder.validateAndFixTerrain();
    AreaBuilder.recalculateAreas();
    TerrainBuilder.storeWaterData();
    generateSnow(iWidth, iHeight);
    dumpStartSectors(startSectors);
    dumpContinents(iWidth, iHeight);
    dumpTerrain(iWidth, iHeight);
    dumpElevation(iWidth, iHeight);
    dumpRainfall(iWidth, iHeight);
    dumpBiomes(iWidth, iHeight);
    dumpFeatures(iWidth, iHeight);
    dumpPermanentSnow(iWidth, iHeight);
    generateResources(iWidth, iHeight, iNumPlayers1, iNumPlayers2, extraMapInfo);
    startPositions = assignStartPositions(iNumPlayers1, iNumPlayers2, westStartRegion, eastStartRegion, iStartSectorRows, iStartSectorCols, startSectors);
    generateDiscoveries(iWidth, iHeight, startPositions);
    dumpResources(iWidth, iHeight);
    FertilityBuilder.recalculate(); // Must be after features are added.
    let avgDistanceBetweenPoints = 3;
    let normalizedRangeSmoothing = 2;
    let poisson = TerrainBuilder.generatePoissonMap(seed, avgDistanceBetweenPoints, normalizedRangeSmoothing);
    let poissonPred = (val) => {
        return val >= 1 ? "*" : " ";
    };
    dumpNoisePredicate(iWidth, iHeight, poisson, poissonPred);
    assignAdvancedStartRegions();
}
// Register listeners.
engine.on('RequestMapInitData', requestMapData);
engine.on('GenerateMap', generateMap);
console.log("Loaded randomContinents.js");
function createLandmasses(iWidth, iHeight, westStartRegion, eastStartRegion, iStartSectorRows, iStartSectorCols, startSectors, mapInfo) {
    let iWidth2 = iWidth / 2;
    FractalBuilder.create(globals.g_LandmassFractal, iWidth, iHeight, 3, 0);
    let waterPercentContinents = globals.g_WaterPercent; // Total of 35%
    let iWaterHeight = FractalBuilder.getHeightFromPercent(globals.g_LandmassFractal, waterPercentContinents);
    let iBuffer = Math.max(2, Math.floor(iHeight / 20));

    // Create random borders between the continents
    let borderStart = createRandomBorder(0, iHeight, iWidth / 12, 3);
    let borderMiddle = createRandomBorder(iWidth2, iHeight, iWidth / 12, 3);

    // Generate base shape of continents
    for (let iY = 0; iY < iHeight; iY++) {
        for (let iX = 0; iX < iWidth; iX++) {
            let curMapInfo = {
                terrain: globals.g_OceanTerrain,
                continent: 1,
                distFromContinent0: 0,
                distFromContinent1: 0,
                distFromContinent2: 0
            };

            // Get the value from the fractal
            let iPlotHeight = utilities.getHeightAdjustingForStartSector(iX, iY, iWaterHeight, globals.g_FractalWeight, 0, globals.g_StartSectorWeight, westStartRegion, eastStartRegion, iStartSectorRows, iStartSectorCols, startSectors);
            iPlotHeight -= iWaterHeight * (globals.g_FractalWeight + globals.g_StartSectorWeight);
            iPlotHeight = (iPlotHeight * 1) / (1 + Math.abs(iPlotHeight * 0.5));
            iPlotHeight += iWaterHeight * (globals.g_FractalWeight + globals.g_StartSectorWeight);

            let distStart = Math.max(0, getBorderDistanceX(borderStart, iX, iY, iWidth) - 2);
            let distMiddle = Math.max(0, getBorderDistanceX(borderMiddle, iX, iY, iWidth) - 2);
            let falloffX = 15 / iWidth;
            iPlotHeight -= 5.0 / Math.pow(1.0 + distStart * falloffX, 1.7);
            iPlotHeight -= 5.0 / Math.pow(1.0 + distMiddle * falloffX, 1.7);

            curMapInfo.continent = getContinentFromBorders(borderStart, borderMiddle, iX, iY, iWidth) ? 0 : 2;

            // Finally see whether or not this stays as Land or has too low a score and drops back to water
            if (iPlotHeight >= iWaterHeight * (globals.g_FractalWeight + globals.g_StartSectorWeight)) {
                curMapInfo.terrain = globals.g_FlatTerrain;
            }

            mapInfo[iX + iY * iWidth] = curMapInfo;
        }
    }

    // Randomly extend land to remove fractal patterns
    for (let test = 0; test < 30; test++) {
        for (let iY = 0; iY < iHeight; iY++) {
            for (let iX = 0; iX < iWidth; iX++) {
                //if (isLandSpotValid(iX, iY, iWidth, seed)) {
                let count = adjacentLandCount(iX, iY, iWidth, iHeight, mapInfo);

                let distStart = Math.max(0, getBorderDistanceX(borderStart, iX, iY, iWidth) - 2);
                let distMiddle = Math.max(0, getBorderDistanceX(borderMiddle, iX, iY, iWidth) - 2);
                let falloffX = 28.0 / iWidth;
                let minDistX = Math.min(distStart, distMiddle);
                let randFacX = 4.0 / Math.pow(1 + minDistX * falloffX, 2);
                if (randFacX > random01()) {
                    continue;
                }

                let isValid = false;
                isValid = isValid || (count == 1 && TerrainBuilder.getRandomNumber(256, "Shallow Water Scater Scatter1") == 0);
                isValid = isValid || (count == 2 && TerrainBuilder.getRandomNumber(48, "Shallow Water Scater Scatter1") == 0);
                isValid = isValid || (count == 3 && TerrainBuilder.getRandomNumber(16, "Shallow Water Scater Scatter1") == 0);
                isValid = isValid || (count == 4 && TerrainBuilder.getRandomNumber(24, "Shallow Water Scater Scatter1") == 0);
                isValid = isValid || (count == 5 && TerrainBuilder.getRandomNumber(8, "Shallow Water Scater Scatter1") == 0);
                isValid = isValid || (count == 6 && TerrainBuilder.getRandomNumber(4, "Shallow Water Scater Scatter1") == 0);
                if (isValid) {
                    mapInfo[iX + iY * iWidth].terrain = globals.g_FlatTerrain;
                }
                //}
            }
        }
    }

    // Homeland Islands
    let totalTiles = iWidth * iHeight;
    let islandPlacementCount = totalTiles / 140;
    for (let test = 0; test < islandPlacementCount; test++) {
        let randStartSize = 4 + TerrainBuilder.getRandomNumber(6, "rand start size");
        let randExtraSize = 4 + TerrainBuilder.getRandomNumber(12, "rand extra size");
        let randSizeRatio = TerrainBuilder.getRandomNumber(randExtraSize, "rand XY ratio");
        let sizeX = randStartSize + randSizeRatio;
        let sizeY = randStartSize + (randExtraSize - randSizeRatio);
        let posX = TerrainBuilder.getRandomNumber(iWidth, "rand pos X");
        let posY = -sizeY + TerrainBuilder.getRandomNumber(sizeY + iHeight, "rand pos Y");

        placeIslandsInRegion(posX, posY, iWidth, iHeight, sizeX, sizeY, borderStart, borderMiddle, mapInfo, 0);
    }

    // Calculate distance field for continent 0 and 2
    for (let test = 0; test < 25; test++) {
        for (let iY = 0; iY < iHeight; iY++) {
            for (let iX = 0; iX < iWidth; iX++) {

                let minNumber0 = 100;
                let minNumber2 = 100;
                for (let iDirection = 0; iDirection < DirectionTypes.NUM_DIRECTION_TYPES; iDirection++) {
                    let iIndex = GameplayMap.getIndexFromXY(iX, iY);
                    let iLocation = GameplayMap.getLocationFromIndex(iIndex);
                    let iAdjacentX = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).x;
                    let iAdjacentY = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).y;
                    if (iAdjacentX < 0 || iAdjacentY < 0 || iAdjacentX >= iWidth || iAdjacentY >= iHeight) {
                        continue;
                    }
                    let curMapInfo = mapInfo[iAdjacentX + iAdjacentY * iWidth];
                    minNumber0 = Math.min(minNumber0, curMapInfo.distFromContinent0);
                    minNumber2 = Math.min(minNumber2, curMapInfo.distFromContinent2);
                }

                if (mapInfo[iX + iY * iWidth].terrain === globals.g_OceanTerrain || mapInfo[iX + iY * iWidth].continent === 2) {
                    mapInfo[iX + iY * iWidth].distFromContinent0 = minNumber0 + 1;
                }
                if (mapInfo[iX + iY * iWidth].terrain === globals.g_OceanTerrain || mapInfo[iX + iY * iWidth].continent === 0) {
                    mapInfo[iX + iY * iWidth].distFromContinent2 = minNumber2 + 1;
                }
            }
        }
    }

    // Islands at border
    totalTiles = iWidth * iHeight;
    islandPlacementCount = totalTiles / 100;
    for (let test = 0; test < islandPlacementCount; test++) {
        let randStartSize = 4 + TerrainBuilder.getRandomNumber(6, "rand start size");
        let randExtraSize = 4 + TerrainBuilder.getRandomNumber(12, "rand extra size");
        let randSizeRatio = TerrainBuilder.getRandomNumber(randExtraSize, "rand XY ratio");
        let sizeX = randStartSize + randSizeRatio;
        let sizeY = randStartSize + (randExtraSize - randSizeRatio);
        let posX = TerrainBuilder.getRandomNumber(iWidth, "rand pos X");
        let posY = -sizeY + TerrainBuilder.getRandomNumber(sizeY + iHeight, "rand pos Y");

        placeIslandsInRegion(posX, posY, iWidth, iHeight, sizeX, sizeY, borderStart, borderMiddle, mapInfo, 1);
    }

    // Calculate distance field for distant islands
    for (let test = 0; test < 20; test++) {
        for (let iY = 0; iY < iHeight; iY++) {
            for (let iX = 0; iX < iWidth; iX++) {

                let minNumber1 = 100;
                for (let iDirection = 0; iDirection < DirectionTypes.NUM_DIRECTION_TYPES; iDirection++) {
                    let iIndex = GameplayMap.getIndexFromXY(iX, iY);
                    let iLocation = GameplayMap.getLocationFromIndex(iIndex);
                    let iAdjacentX = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).x;
                    let iAdjacentY = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).y;
                    if (iAdjacentX < 0 || iAdjacentY < 0 || iAdjacentX >= iWidth || iAdjacentY >= iHeight) {
                        continue;
                    }
                    let curMapInfo = mapInfo[iAdjacentX + iAdjacentY * iWidth];
                    minNumber1 = Math.min(minNumber1, curMapInfo.distFromContinent1);
                }

                if (mapInfo[iX + iY * iWidth].terrain === globals.g_OceanTerrain || mapInfo[iX + iY * iWidth].continent !== 1) {
                    mapInfo[iX + iY * iWidth].distFromContinent1 = minNumber1 + 1;
                }
            }
        }
    }

    // Calculate random coast tiles
    for (let iY = 0; iY < iHeight; iY++) {
        for (let iX = 0; iX < iWidth; iX++) {
            let curMapInfo = mapInfo[iX + iY * iWidth];

            let distDif01 = Math.abs(curMapInfo.distFromContinent0 - curMapInfo.distFromContinent1);
            let distDif02 = Math.abs(curMapInfo.distFromContinent0 - curMapInfo.distFromContinent2);
            let distDif12 = Math.abs(curMapInfo.distFromContinent1 - curMapInfo.distFromContinent2);
            let minDistDif1 = Math.min(distDif01, distDif12);

            if (curMapInfo.terrain === globals.g_OceanTerrain && minDistDif1 > 2 && (distDif02 > 2 || curMapInfo.distFromContinent1 <= distDif02) && isAdjacentToShallowWater(iX, iY, iWidth, iHeight, mapInfo) && TerrainBuilder.getRandomNumber(2, "Shallow Water Scater Scatter") == 0) {
                mapInfo[iX + iY * iWidth].terrain = globals.g_CoastTerrain;
            }
        }
    }

    // Random top and bottom spacing
    for (let iY = 0; iY < iHeight; iY++) {
        for (let iX = 0; iX < iWidth; iX++) {
            let iRandom = TerrainBuilder.getRandomNumber(iBuffer, "Random Top/Bottom Edges");
            if (iY < iRandom + 1 || iY >= iHeight - (iRandom + 1)) {
                mapInfo[iX + iY * iWidth].terrain = globals.g_OceanTerrain;
            }
        }
    }

    // Set engine land and ocean values from mapInfo
    for (let iY = 0; iY < iHeight; iY++) {
        for (let iX = 0; iX < iWidth; iX++) {
            let curMapInfo = mapInfo[iX + iY * iWidth];

            if (curMapInfo.terrain === globals.g_FlatTerrain) {
                TerrainBuilder.addPlotTag(iX, iY, curMapInfo.continent == 1 ? PlotTags.PLOT_TAG_ISLAND : (curMapInfo.continent == 0 ? PlotTags.PLOT_TAG_WEST_LANDMASS : PlotTags.PLOT_TAG_EAST_LANDMASS));
            } else if (curMapInfo.terrain === globals.g_OceanTerrain) {
                TerrainBuilder.addPlotTag(iX, iY, curMapInfo.continent <= 1 ? PlotTags.PLOT_TAG_WEST_WATER : PlotTags.PLOT_TAG_EAST_WATER);
            }
            TerrainBuilder.setTerrainType(iX, iY, curMapInfo.terrain);
        }
    }
}

export function placeIslandsInRegion(iX, iY, iWidth, iHeight, sizeX, sizeY, borderStart, borderMiddle, mapInfo, placementType) {

    let weightSum = 0.1;
    let weightCount = 0.1;
    let validArray = new Array(sizeX * sizeY);
    for (let y = 0; y < sizeY; y++) {
        for (let x = 0; x < sizeX; x++) {
            let isSpotValid = true;
            let X = x + iX;
            let Y = y + iY;
            if (X < 0) { X += iWidth; }
            if (X >= iWidth) { X = X % iWidth; }
            if (Y < 0 || Y >= iHeight) { isSpotValid = false; }

            if (!isSpotValid) {
                validArray[x + y * sizeX] = false;
            } else {
                weightCount++;
                let isNoContinentNear = isNoContinentAround(X, Y, iWidth, iHeight, mapInfo);
                if (isNoContinentNear) {
                    weightSum += 1.0;
                }
                validArray[x + y * sizeX] = isNoContinentNear;
            }
        }
    }

    let validRatio = weightSum / parseFloat(weightCount);
    if (validRatio < 0.25) { return; }

    FractalBuilder.create(globals.g_LandmassFractal, iWidth, iHeight, 4 + TerrainBuilder.getRandomNumber(2, "fractal offset"), 0);
    let waterPercentIslands = 50 + TerrainBuilder.getRandomNumber(30, "extra island water percent");
    let iWaterHeightIsland = FractalBuilder.getHeightFromPercent(globals.g_LandmassFractal, waterPercentIslands);

    for (let y = 0; y < sizeY; y++) {
        for (let x = 0; x < sizeX; x++) {
            if (!validArray[x + y * sizeX]) {
                continue;
            }

            let X = x + iX;
            let Y = y + iY;
            if (X < 0) { X += iWidth; }
            if (X >= iWidth) { X = X % iWidth; }

            let iPlotHeight = FractalBuilder.getHeight(globals.g_LandmassFractal, X, Y);

            iPlotHeight -= iWaterHeightIsland;
            iPlotHeight = (iPlotHeight * 1) / (1 + Math.abs(iPlotHeight * 0.5));
            iPlotHeight += iWaterHeightIsland;

            let difX = Math.abs(x - sizeX / 2.0) / parseFloat(sizeX / 2.0);
            let difY = Math.abs(y - sizeY / 2.0) / parseFloat(sizeY / 2.0);
            let difXY = difX * difX + difY * difY;
            let difNorm = Math.pow(difXY, 1.0);
            iPlotHeight -= difNorm * 0.4;

            if (placementType == 0) {
                let distStart = Math.max(0, getBorderDistanceX(borderStart, X, Y, iWidth) - 2);
                let distMiddle = Math.max(0, getBorderDistanceX(borderMiddle, X, Y, iWidth) - 2);
                let falloffX = 12.8 / iWidth;
                iPlotHeight -= 4.0 / Math.pow(1.0 + distStart * falloffX, 1.7);
                iPlotHeight -= 4.0 / Math.pow(1.0 + distMiddle * falloffX, 1.7);

                if (iPlotHeight > iWaterHeightIsland) {
                    mapInfo[X + Y * iWidth].terrain = globals.g_FlatTerrain;
                }
            } else {
                let minDist = 3 + (TerrainBuilder.getRandomNumber(10, "continent island spacing") < 8 ? 1 : 0);
                if (iPlotHeight > iWaterHeightIsland && mapInfo[X + Y * iWidth].distFromContinent0 > minDist && mapInfo[X + Y * iWidth].distFromContinent2 > minDist) {
                    mapInfo[X + Y * iWidth].terrain = globals.g_FlatTerrain;
                    mapInfo[X + Y * iWidth].continent = 1;
                }
            }


        }
    }
}

export function adjacentLandCount(iX, iY, iWidth, iHeight, mapInfo) {
    let count = 0;
    for (let iDirection = 0; iDirection < DirectionTypes.NUM_DIRECTION_TYPES; iDirection++) {
        let iIndex = GameplayMap.getIndexFromXY(iX, iY);
        let iLocation = GameplayMap.getLocationFromIndex(iIndex);
        let iAdjacentX = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).x;
        let iAdjacentY = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).y;
        if (iAdjacentX < 0 || iAdjacentY < 0 || iAdjacentX >= iWidth || iAdjacentY >= iHeight)
            continue;
        if (mapInfo[iAdjacentX + iAdjacentY * iWidth].terrain === globals.g_FlatTerrain) {
            count++;
        }
    }
    return count;
}

export function isNoContinentAround(iX, iY, iWidth, iHeight, mapInfo, terrain = globals.g_OceanTerrain) {
    let count = 0;
    if (mapInfo[iX + iY * iWidth].terrain === globals.g_OceanTerrain) {
        for (let iDirection = 0; iDirection < DirectionTypes.NUM_DIRECTION_TYPES; iDirection++) {
            let iIndex = GameplayMap.getIndexFromXY(iX, iY);
            let iLocation = GameplayMap.getLocationFromIndex(iIndex);
            let iAdjacentX = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).x;
            let iAdjacentY = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).y;
            if (iAdjacentX < 0 || iAdjacentY < 0 || iAdjacentX >= iWidth || iAdjacentY >= iHeight) {
                count++;
                continue;
            }

            if (mapInfo[iAdjacentX + iAdjacentY * iWidth].terrain === globals.g_OceanTerrain || mapInfo[iAdjacentX + iAdjacentY * iWidth].continent === 1) {
                count++;
            }
        }
    }
    return count == 6;
}

export function isAdjacentToShallowWater(iX, iY, iWidth, iHeight, mapInfo) {
    let count = 0;
    for (let iDirection = 0; iDirection < DirectionTypes.NUM_DIRECTION_TYPES; iDirection++) {
        let iIndex = GameplayMap.getIndexFromXY(iX, iY);
        let iLocation = GameplayMap.getLocationFromIndex(iIndex);
        let iAdjacentX = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).x;
        let iAdjacentY = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).y;
        if (iAdjacentX < 0 || iAdjacentY < 0 || iAdjacentX >= iWidth || iAdjacentY >= iHeight) {
            continue;
        }

        if (mapInfo[iAdjacentX + iAdjacentY * iWidth].terrain === globals.g_CoastTerrain || mapInfo[iAdjacentX + iAdjacentY * iWidth].terrain === globals.g_FlatTerrain) {
            count++;
        }
    }
    return count > 0;
}

//# sourceMappingURL=file:///base-standard/maps/fractal.js.map
